
# dateToIssue.py
# Andrew Davison, March 2025, ad@coe.psu.ac.th

'''
> python dateToIssue.py
Enter a date (dd MMM yyyy or MMM dd yyyy): 24 Mar 1972
Issue on 18 Mar 1972, no. 104
Not a publication date.
Previous issue is 104 on 18 Mar 1972

> python dateToIssue.py
Enter a date (dd MMM yyyy or MMM dd yyyy): 24 Mar 2021
After last issue on 1 Mar 1975
'''

from datetime import datetime, timedelta


issueDates = [   # day 3-letter-month year
  ("28 Mar 1970", 1),
  ("2 Jan 1971", 41), ("18 Sep 1971", 78),
  ("18 Mar 1972", 104), ("16 Sep 1972", 130),
  ("17 Mar 1973", 156), ("15 Sep 1973", 182),
  ("16 Mar 1974", 208), ("14 Sep 1974", 234)
]

LAST_ISSUE = ("1 Mar 1975", 258)
lastDate = datetime.strptime(LAST_ISSUE[0], "%d %b %Y")


def parseDate(dateStr):
  formats = ["%d %b %Y", "%b %d %Y", "%d %B %Y", "%B %d %Y"]
  for fmt in formats:
    try:
      return datetime.strptime(dateStr, fmt)
    except ValueError:
      pass
  raise ValueError("Invalid date format")


def getStartDate(userDate):
  # find largest date smaller than the user date
  for dateStr, issue in reversed(issueDates):
    date = datetime.strptime(dateStr, "%d %b %Y")
    if date <= userDate:
      print(f"Issue on {dateStr}, no. {issue}")
      return date, issue

  print("Before the first issue on", issueDates[0][0])
  return None, None



def calcIssue(startDate, startIssue, userDate):
  count = 0
  currentDate = startDate
  while currentDate < userDate:
    count += 1
    prevDate = currentDate
    currentDate += timedelta(days=7)

  if currentDate > userDate:
    print("Not a publication date.")
    print(f"Previous issue is {startIssue + count - 1} on {prevDate.strftime('%d %b %Y')}")
  else:
    print(f"The issue is: {startIssue + count}")


# ---------------------------

# Read date from user input
userDateStr = input("Enter a date (dd MMM yyyy or MMM dd yyyy): ")
userDate = parseDate(userDateStr)

if userDate > lastDate:
  print("After last issue on", LAST_ISSUE[0])
else: 
  startDate, startIssue = getStartDate(userDate)
  if startDate != None:
    calcIssue(startDate, startIssue, userDate)

